import {
  Directive,
  ElementRef,
  Input,
  Renderer2,
} from "@angular/core";
import {
  isFunction,
  isTruthy,
  isTruthyObject,
} from "../../tools";

@Directive({
  selector: "[solidifyAutoScrollIntoView]",
})

export class AutoScrollIntoViewDirective {

  private _autoScrollIntoView: boolean = false;

  @Input("solidifyAutoScrollIntoView")
  set autoScrollIntoView(value: boolean) {
    const valueBoolean = isTruthy(value);
    if (this._autoScrollIntoView !== valueBoolean) {
      this._autoScrollIntoView = valueBoolean;
      this._onAutoScrollIntoViewChange();
    }
  }

  constructor(private readonly _elementRef: ElementRef<HTMLElement>,
              private readonly _renderer: Renderer2) {
  }

  private _onAutoScrollIntoViewChange(): void {
    if (this._autoScrollIntoView) {
      const nativeElement = this._elementRef.nativeElement;
      if (isTruthyObject(nativeElement) && isFunction(nativeElement.scrollIntoView)) {
        nativeElement.scrollIntoView({behavior: "smooth", block: "nearest"});
      }
    }
  }

}
